using System;
using System.Text;
using NUnit.Framework;

namespace NUnit.Extensions.Converter
{
	public class ConverterContext 
	{
		public int ExpansionLevel = 0;
		public string Indent = "";
		public void BeginExpansion()
		{
			ExpansionLevel--;
			Indent += "  ";
		}
		public void EndExpansion()
		{
			Indent = Indent.Substring(0, Indent.Length - 2);
			ExpansionLevel++;
		}
	}
	public abstract class Converter
	{
		abstract public string ToString(object o);
		abstract public int ExpansionLevel
		{
			set;
		}
		abstract public ConverterContext Context
		{
			get;
		}
		public static string INVISIBLE = "__NUNIT_CONVERTER_INVISIBLE__";
		protected Converter outermostConverter;

		public virtual void SetOutermostConverter(Converter outermostConverter)
		{
			this.outermostConverter = outermostConverter;
		}
		protected string DelegateToString(object o)
		{
			if (NotVisible(o))
				return Converter.INVISIBLE;
			return o is string ? "" + o : outermostConverter.ToString(o);
		}
		protected string DelegateToString(char c)
		{
			return  outermostConverter.ToString(c);
		}
		protected string DelegateToString(int i)
		{
			return  outermostConverter.ToString(i);
		}
		protected string DelegateToString(long l)
		{
			return  outermostConverter.ToString(l);
		}
		protected string DelegateToString(double d)
		{
			return  outermostConverter.ToString(d);
		}
		protected string DelegateToString(float f)
		{
			return  outermostConverter.ToString(f);
		}
		public string ToString(object o, int expansionLevel)
		{
			if (expansionLevel == -1)
				return ToString(o);

			int save = Context.ExpansionLevel;
			Context.ExpansionLevel = expansionLevel;
			string result = ToString(o);
			Context.ExpansionLevel = save;
			return result;
		}
		public void Print(object o)
		{
			Print(o, -1);
		}
		public void Print(object o, int expansionLevel)
		{
			Console.Write(ToString(o, expansionLevel));
		}
		public void Println(object o)
		{
			Println(o, -1);
		}
		public void Println(object o, int expansionLevel)
		{
			Console.WriteLine(ToString(o, expansionLevel));
		}
		private void AssertEquals(string message, string expected, string actual)
		{
			Assert.AreEqual(new StringWrapper(expected), new StringWrapper(actual), message);
		}
		public void AssertEquals(object expected, object actual)
		{
			AssertEquals(string.Empty, expected, actual);
		}
		public void AssertEquals(string message, object expected, object actual)
		{
			AssertEquals(message, expected, actual, -1);
		}
		public void AssertEquals(object expected, object actual, int expansionLevel)
		{
			AssertEquals(null, expected, actual, expansionLevel);
		}
		public void AssertEquals(string message, object expected, object actual, int expansionLevel)
		{
			AssertEquals(message, ToString(expected, expansionLevel), ToString(actual, expansionLevel));
		}
		public void AssertEqualsIgnoreOrder(object expected, object actual)
		{
			AssertEqualsIgnoreOrder(null, expected, actual);
		}
		public void AssertEqualsIgnoreOrder(string message, object expected, object actual)
		{
			AssertEqualsIgnoreOrder(message, expected, actual, -1);
		}
		public void AssertEqualsIgnoreOrder(object expected, object actual, int expansionLevel)
		{
			AssertEqualsIgnoreOrder(null, expected, actual, expansionLevel);
		}
		public void AssertEqualsIgnoreOrder(string massage, object expected, object actual, int expansionLevel)
		{
			Sorter sorter = new Sorter(outermostConverter);
			AssertEquals(massage, sorter.Sort(expected), sorter.Sort(actual), expansionLevel);
		}
		public void AssertSame(object expected, object actual)
		{
			AssertSame(null, expected, actual);
		}
		public void AssertSame(string message, object expected, object actual)
		{
			AssertSame(message, expected, actual, -1);
		}
		public void AssertSame(object expected, object actual, int expansionLevel)
		{
			AssertSame(null, expected, actual, expansionLevel);
		}
		public void AssertSame(string message, object expected, object actual, int expansionLevel) 
		{
			if (expected == actual)
				return;
			FailNotSame(message, expected, actual, expansionLevel);
		}
		private void FailNotSame(string message, object expected, object actual, int expansionLevel) 
		{
			StringBuilder buf = new StringBuilder();
			if (message != null)
				buf.Append(message + " ");
			buf.Append("expected same:<");
			buf.Append(ToString(expected, expansionLevel));
			buf.Append("> was not:<");
			buf.Append(ToString(actual, expansionLevel));
			buf.Append(">");
			Assert.Fail(buf.ToString());
		}
		public virtual void AddEvaluator(Evaluator evaluator)
		{
			throw new NotSupportedException();
		}
		static public bool IsVisible(object o)
		{
			return !NotVisible(o);
		}
		static public bool NotVisible(object o)
		{
			return (object)o == (object)INVISIBLE;
		}
	}
}
